@@ -1,21 +1,14 @@
 #![doc(html_root_url = "https://doc.rust-lang.org/nightly/")]
-
 #![feature(bool_to_option)]
-#![feature(box_patterns)]
 #![feature(core_intrinsics)]
 #![feature(crate_visibility_modifier)]
 #![feature(drain_filter)]
 #![feature(in_band_lifetimes)]
-#![feature(libc)]
 #![feature(nll)]
 #![feature(proc_macro_internals)]
-#![feature(proc_macro_quote)]
-#![feature(rustc_private)]
-#![feature(slice_patterns)]
 #![feature(specialization)]
 #![feature(stmt_expr_attributes)]
-
-#![recursion_limit="256"]
+#![recursion_limit = "256"]
 
 extern crate libc;
 extern crate proc_macro;
@@ -40,7 +33,7 @@ pub mod locator;
 pub fn validate_crate_name(
     sess: Option<&rustc::session::Session>,
     s: &str,
-    sp: Option<syntax_pos::Span>
+    sp: Option<rustc_span::Span>,
 ) {
     let mut err_count = 0;
     {
@@ -56,8 +49,12 @@ pub fn validate_crate_name(
             say("crate name must not be empty");
         }
         for c in s.chars() {
-            if c.is_alphanumeric() { continue }
-            if c == '_'  { continue }
+            if c.is_alphanumeric() {
+                continue;
+            }
+            if c == '_' {
+                continue;
+            }
             say(&format!("invalid character `{}` in crate name: `{}`", c, s));
         }
     }
