@@ -1,20 +1,20 @@
+use rustc_ast::token::{self, Token, TokenKind};
+use rustc_ast::util::comments;
 use rustc_data_structures::sync::Lrc;
-use rustc_errors::{FatalError, DiagnosticBuilder};
-use rustc_lexer::Base;
+use rustc_errors::{error_code, DiagnosticBuilder, FatalError};
 use rustc_lexer::unescape;
-use syntax::token::{self, Token, TokenKind};
-use syntax::sess::ParseSess;
-use syntax::util::comments;
-use syntax_pos::symbol::{sym, Symbol};
-use syntax_pos::{BytePos, Pos, Span};
+use rustc_lexer::Base;
+use rustc_session::parse::ParseSess;
+use rustc_span::symbol::{sym, Symbol};
+use rustc_span::{BytePos, Pos, Span};
 
+use log::debug;
 use std::char;
 use std::convert::TryInto;
-use log::debug;
 
 mod tokentrees;
-mod unicode_chars;
 mod unescape_error_reporting;
+mod unicode_chars;
 use unescape_error_reporting::{emit_unescape_error, push_escaped_char};
 
 #[derive(Clone, Debug)]
@@ -41,12 +41,14 @@ pub struct StringReader<'a> {
 }
 
 impl<'a> StringReader<'a> {
-    pub fn new(sess: &'a ParseSess,
-               source_file: Lrc<syntax_pos::SourceFile>,
-               override_span: Option<Span>) -> Self {
+    pub fn new(
+        sess: &'a ParseSess,
+        source_file: Lrc<rustc_span::SourceFile>,
+        override_span: Option<Span>,
+    ) -> Self {
         if source_file.src.is_none() {
-            sess.span_diagnostic.bug(&format!("cannot lex `source_file` without source: {}",
-                                              source_file.name));
+            sess.span_diagnostic
+                .bug(&format!("cannot lex `source_file` without source: {}", source_file.name));
         }
 
         let src = (*source_file.src.as_ref().unwrap()).clone();
@@ -78,7 +80,6 @@ impl<'a> StringReader<'a> {
         sr
     }
 
-
     fn mk_sp(&self, lo: BytePos, hi: BytePos) -> Span {
         self.override_span.unwrap_or_else(|| Span::with_root_ctxt(lo, hi))
     }
@@ -137,7 +138,6 @@ impl<'a> StringReader<'a> {
         self.sess.span_diagnostic.struct_span_err(sp, m).emit();
     }
 
-
     /// Report a fatal error spanning [`from_pos`, `to_pos`).
     fn fatal_span_(&self, from_pos: BytePos, to_pos: BytePos, m: &str) -> FatalError {
         self.fatal_span(self.mk_sp(from_pos, to_pos), m)
@@ -148,15 +148,22 @@ impl<'a> StringReader<'a> {
         self.err_span(self.mk_sp(from_pos, to_pos), m)
     }
 
-    fn struct_span_fatal(&self, from_pos: BytePos, to_pos: BytePos, m: &str)
-        -> DiagnosticBuilder<'a>
-    {
+    fn struct_span_fatal(
+        &self,
+        from_pos: BytePos,
+        to_pos: BytePos,
+        m: &str,
+    ) -> DiagnosticBuilder<'a> {
         self.sess.span_diagnostic.struct_span_fatal(self.mk_sp(from_pos, to_pos), m)
     }
 
-    fn struct_fatal_span_char(&self, from_pos: BytePos, to_pos: BytePos, m: &str, c: char)
-        -> DiagnosticBuilder<'a>
-    {
+    fn struct_fatal_span_char(
+        &self,
+        from_pos: BytePos,
+        to_pos: BytePos,
+        m: &str,
+        c: char,
+    ) -> DiagnosticBuilder<'a> {
         let mut m = m.to_string();
         m.push_str(": ");
         push_escaped_char(&mut m, c);
@@ -165,13 +172,9 @@ impl<'a> StringReader<'a> {
     }
 
     /// Turns simple `rustc_lexer::TokenKind` enum into a rich
-    /// `libsyntax::TokenKind`. This turns strings into interned
+    /// `librustc_ast::TokenKind`. This turns strings into interned
     /// symbols and runs additional validation.
-    fn cook_lexer_token(
-        &self,
-        token: rustc_lexer::TokenKind,
-        start: BytePos,
-    ) -> TokenKind {
+    fn cook_lexer_token(&self, token: rustc_lexer::TokenKind, start: BytePos) -> TokenKind {
         match token {
             rustc_lexer::TokenKind::LineComment => {
                 let string = self.str_from(start);
@@ -202,9 +205,7 @@ impl<'a> StringReader<'a> {
                 }
 
                 let tok = if is_doc_comment {
-                    self.forbid_bare_cr(start,
-                                        string,
-                                        "bare CR not allowed in block doc-comment");
+                    self.forbid_bare_cr(start, string, "bare CR not allowed in block doc-comment");
                     token::DocComment(Symbol::intern(string))
                 } else {
                     token::Comment
@@ -219,8 +220,7 @@ impl<'a> StringReader<'a> {
                 if is_raw_ident {
                     ident_start = ident_start + BytePos(2);
                 }
-                // FIXME: perform NFKC normalization here. (Issue #2253)
-                let sym = self.symbol_from(ident_start);
+                let sym = nfc_normalize(self.str_from(ident_start));
                 if is_raw_ident {
                     let span = self.mk_sp(start, self.pos);
                     if !sym.can_be_raw() {
@@ -236,14 +236,22 @@ impl<'a> StringReader<'a> {
                 let suffix = if suffix_start < self.pos {
                     let string = self.str_from(suffix_start);
                     if string == "_" {
-                        self.sess.span_diagnostic
-                            .struct_span_warn(self.mk_sp(suffix_start, self.pos),
-                                              "underscore literal suffix is not allowed")
-                            .warn("this was previously accepted by the compiler but is \
+                        self.sess
+                            .span_diagnostic
+                            .struct_span_warn(
+                                self.mk_sp(suffix_start, self.pos),
+                                "underscore literal suffix is not allowed",
+                            )
+                            .warn(
+                                "this was previously accepted by the compiler but is \
                                    being phased out; it will become a hard error in \
-                                   a future release!")
-                            .note("for more information, see issue #42326 \
-                                   <https://github.com/rust-lang/rust/issues/42326>")
+                                   a future release!",
+                            )
+                            .note(
+                                "see issue #42326 \
+                                 <https://github.com/rust-lang/rust/issues/42326> \
+                                 for more information",
+                            )
                             .emit();
                         None
                     } else {
@@ -260,11 +268,7 @@ impl<'a> StringReader<'a> {
                 // this is necessary.
                 let lifetime_name = self.str_from(start);
                 if starts_with_number {
-                    self.err_span_(
-                        start,
-                        self.pos,
-                        "lifetimes cannot start with a number",
-                    );
+                    self.err_span_(start, self.pos, "lifetimes cannot start with a number");
                 }
                 let ident = Symbol::intern(lifetime_name);
                 token::Lifetime(ident)
@@ -299,10 +303,8 @@ impl<'a> StringReader<'a> {
 
             rustc_lexer::TokenKind::Unknown => {
                 let c = self.str_from(start).chars().next().unwrap();
-                let mut err = self.struct_fatal_span_char(start,
-                                                          self.pos,
-                                                          "unknown start of token",
-                                                          c);
+                let mut err =
+                    self.struct_fatal_span_char(start, self.pos, "unknown start of token", c);
                 // FIXME: the lexer could be used to turn the ASCII version of unicode homoglyphs,
                 // instead of keeping a table in `check_for_substitution`into the token. Ideally,
                 // this should be inside `rustc_lexer`. However, we should first remove compound
@@ -320,25 +322,22 @@ impl<'a> StringReader<'a> {
         &self,
         start: BytePos,
         suffix_start: BytePos,
-        kind: rustc_lexer::LiteralKind
+        kind: rustc_lexer::LiteralKind,
     ) -> (token::LitKind, Symbol) {
         match kind {
             rustc_lexer::LiteralKind::Char { terminated } => {
                 if !terminated {
-                    self.fatal_span_(start, suffix_start,
-                                     "unterminated character literal".into())
-                        .raise()
+                    self.fatal_span_(start, suffix_start, "unterminated character literal").raise()
                 }
                 let content_start = start + BytePos(1);
                 let content_end = suffix_start - BytePos(1);
                 self.validate_char_escape(content_start, content_end);
                 let id = self.symbol_from_to(content_start, content_end);
                 (token::Char, id)
-            },
+            }
             rustc_lexer::LiteralKind::Byte { terminated } => {
                 if !terminated {
-                    self.fatal_span_(start + BytePos(1), suffix_start,
-                                     "unterminated byte constant".into())
+                    self.fatal_span_(start + BytePos(1), suffix_start, "unterminated byte constant")
                         .raise()
                 }
                 let content_start = start + BytePos(2);
@@ -346,11 +345,10 @@ impl<'a> StringReader<'a> {
                 self.validate_byte_escape(content_start, content_end);
                 let id = self.symbol_from_to(content_start, content_end);
                 (token::Byte, id)
-            },
+            }
             rustc_lexer::LiteralKind::Str { terminated } => {
                 if !terminated {
-                    self.fatal_span_(start, suffix_start,
-                                     "unterminated double quote string".into())
+                    self.fatal_span_(start, suffix_start, "unterminated double quote string")
                         .raise()
                 }
                 let content_start = start + BytePos(1);
@@ -361,9 +359,12 @@ impl<'a> StringReader<'a> {
             }
             rustc_lexer::LiteralKind::ByteStr { terminated } => {
                 if !terminated {
-                    self.fatal_span_(start + BytePos(1), suffix_start,
-                                     "unterminated double quote byte string".into())
-                        .raise()
+                    self.fatal_span_(
+                        start + BytePos(1),
+                        suffix_start,
+                        "unterminated double quote byte string",
+                    )
+                    .raise()
                 }
                 let content_start = start + BytePos(2);
                 let content_end = suffix_start - BytePos(1);
@@ -409,35 +410,35 @@ impl<'a> StringReader<'a> {
                     self.validate_int_literal(base, start, suffix_start);
                     (token::Integer, self.symbol_from_to(start, suffix_start))
                 }
-            },
+            }
             rustc_lexer::LiteralKind::Float { base, empty_exponent } => {
                 if empty_exponent {
                     let mut err = self.struct_span_fatal(
-                        start, self.pos,
-                        "expected at least one digit in exponent"
+                        start,
+                        self.pos,
+                        "expected at least one digit in exponent",
                     );
                     err.emit();
                 }
 
                 match base {
-                    Base::Hexadecimal => {
-                        self.err_span_(start, suffix_start,
-                                       "hexadecimal float literal is not supported")
-                    }
+                    Base::Hexadecimal => self.err_span_(
+                        start,
+                        suffix_start,
+                        "hexadecimal float literal is not supported",
+                    ),
                     Base::Octal => {
-                        self.err_span_(start, suffix_start,
-                                       "octal float literal is not supported")
+                        self.err_span_(start, suffix_start, "octal float literal is not supported")
                     }
                     Base::Binary => {
-                        self.err_span_(start, suffix_start,
-                                       "binary float literal is not supported")
+                        self.err_span_(start, suffix_start, "binary float literal is not supported")
                     }
-                    _ => ()
+                    _ => (),
                 }
 
                 let id = self.symbol_from_to(start, suffix_start);
                 (token::Float, id)
-            },
+            }
         }
     }
 
@@ -448,8 +449,7 @@ impl<'a> StringReader<'a> {
 
     /// Slice of the source text from `start` up to but excluding `self.pos`,
     /// meaning the slice does not include the character `self.ch`.
-    fn str_from(&self, start: BytePos) -> &str
-    {
+    fn str_from(&self, start: BytePos) -> &str {
         self.str_from_to(start, self.pos)
     }
 
@@ -466,8 +466,7 @@ impl<'a> StringReader<'a> {
     }
 
     /// Slice of the source text spanning from `start` up to but excluding `end`.
-    fn str_from_to(&self, start: BytePos, end: BytePos) -> &str
-    {
+    fn str_from_to(&self, start: BytePos, end: BytePos) -> &str {
         &self.src[self.src_index(start)..self.src_index(end)]
     }
 
@@ -476,41 +475,38 @@ impl<'a> StringReader<'a> {
         loop {
             idx = match s[idx..].find('\r') {
                 None => break,
-                Some(it) => idx + it + 1
+                Some(it) => idx + it + 1,
             };
-            self.err_span_(start + BytePos(idx as u32 - 1),
-                           start + BytePos(idx as u32),
-                           errmsg);
+            self.err_span_(start + BytePos(idx as u32 - 1), start + BytePos(idx as u32), errmsg);
         }
     }
 
     fn report_non_started_raw_string(&self, start: BytePos) -> ! {
         let bad_char = self.str_from(start).chars().last().unwrap();
-        self
-            .struct_fatal_span_char(
-                start,
-                self.pos,
-                "found invalid character; only `#` is allowed \
+        self.struct_fatal_span_char(
+            start,
+            self.pos,
+            "found invalid character; only `#` is allowed \
                  in raw string delimitation",
-                bad_char,
-            )
-            .emit();
+            bad_char,
+        )
+        .emit();
         FatalError.raise()
     }
 
     fn report_unterminated_raw_string(&self, start: BytePos, n_hashes: usize) -> ! {
-        let mut err = self.struct_span_fatal(
-            start, start,
-            "unterminated raw string",
-        );
-        err.span_label(
+        let mut err = self.sess.span_diagnostic.struct_span_fatal_with_code(
             self.mk_sp(start, start),
             "unterminated raw string",
+            error_code!(E0748),
         );
+        err.span_label(self.mk_sp(start, start), "unterminated raw string");
 
         if n_hashes > 0 {
-            err.note(&format!("this raw string should be terminated with `\"{}`",
-                                "#".repeat(n_hashes as usize)));
+            err.note(&format!(
+                "this raw string should be terminated with `\"{}`",
+                "#".repeat(n_hashes as usize)
+            ));
         }
 
         err.emit();
@@ -521,10 +517,13 @@ impl<'a> StringReader<'a> {
         match n_hashes.try_into() {
             Ok(n_hashes) => n_hashes,
             Err(_) => {
-                self.fatal_span_(start,
-                                 self.pos,
-                                 "too many `#` symbols: raw strings may be \
-                                  delimited by up to 65535 `#` symbols").raise();
+                self.fatal_span_(
+                    start,
+                    self.pos,
+                    "too many `#` symbols: raw strings may be \
+                                  delimited by up to 65535 `#` symbols",
+                )
+                .raise();
             }
         }
     }
@@ -633,10 +632,19 @@ impl<'a> StringReader<'a> {
             if c != '_' && c.to_digit(base).is_none() {
                 let lo = content_start + BytePos(2 + idx);
                 let hi = content_start + BytePos(2 + idx + c.len_utf8() as u32);
-                self.err_span_(lo, hi,
-                               &format!("invalid digit for a base {} literal", base));
-
+                self.err_span_(lo, hi, &format!("invalid digit for a base {} literal", base));
             }
         }
     }
 }
+
+pub fn nfc_normalize(string: &str) -> Symbol {
+    use unicode_normalization::{is_nfc_quick, IsNormalized, UnicodeNormalization};
+    match is_nfc_quick(string.chars()) {
+        IsNormalized::Yes => Symbol::intern(string),
+        _ => {
+            let normalized_str: String = string.chars().nfc().collect();
+            Symbol::intern(&normalized_str)
+        }
+    }
+}
