@@ -8,28 +8,16 @@
 //! LLVM.
 
 #![doc(html_root_url = "https://doc.rust-lang.org/nightly/")]
-
-#![feature(box_syntax)]
+#![feature(bool_to_option)]
 #![feature(nll)]
-#![feature(rustc_attrs)]
-#![feature(slice_patterns)]
-#![feature(step_trait)]
-
-#![deny(rust_2018_idioms)]
-#![deny(internal)]
-#![deny(unused_lifetimes)]
-
-#[macro_use] extern crate log;
-
-#[allow(unused_extern_crates)]
-extern crate serialize as rustc_serialize; // used by deriving
-
-// See librustc_cratesio_shim/Cargo.toml for a comment explaining this.
-#[allow(unused_extern_crates)]
-extern crate rustc_cratesio_shim;
 
 #[macro_use]
-extern crate rustc_data_structures;
+extern crate log;
 
 pub mod abi;
 pub mod spec;
+
+/// Requirements for a `StableHashingContext` to be used in this crate.
+/// This is a hack to allow using the `HashStable_Generic` derive macro
+/// instead of implementing everything in librustc.
+pub trait HashStableContext {}
