@@ -2,12 +2,7 @@
 //~^ WARN the feature `const_generics` is incomplete and may cause the compiler to crash
 
 fn f<T: Copy, const N: usize>(x: T) -> [T; N] {
-    [x; {N}]
-    //~^ ERROR array lengths can't depend on generic parameters
-}
-
-fn g<T, const N: usize>(x: T) -> [T; N] {
-    [x; {N}]
+    [x; N]
     //~^ ERROR array lengths can't depend on generic parameters
 }
 
